<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\controllers;

// Files of the project
use mirzaev\arming_bot\controllers\core,
	mirzaev\arming_bot\models\entry,
	mirzaev\arming_bot\models\category,
	mirzaev\arming_bot\models\product;

/**
 * Controller of catalog
 *
 * @package mirzaev\arming_bot\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class catalog extends core
{
	/**
	 * Registry of errors
	 */
	protected array $errors = [
		'session' => [],
		'account' => [],
		'catalog' => []
	];

	/**
	 * Catalog
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function index(array $parameters = []): ?string
	{
		// Initializing identifier of a category
		preg_match('/[\d]+/', $parameters['identifier'] ?? '', $matches);
		$identifier = $matches[0] ?? null;

		// Initializint the buffer of respnse
		$html = [];

		if (!empty($parameters['identifier'])) {
			// Передана категория (идентификатор)

			// Инициализация актуальной категории
			$category = category::_read('d.identifier == @identifier', parameters: ['identifier' => (int) $identifier], errors: $this->errors['catalog']);

			if ($category instanceof category) {
				// Found the category

				// Поиск категорий или товаров входящих в актуальную категорию
				$entries = entry::search(
					document: $category,
					amount: 30,
					errors: $this->errors['catalog']
				);

				// Объявление буферов категорий и товаров (важно - в единственном числе, по параметру из базы данных)
				$category = $product = [];

				foreach ($entries as $entry) {
					// Перебор вхождений

					// Запись массивов категорий и товаров ($category и $product) в буфер глобальной переменной шаблонизатора
					${$entry->_type}[] = $entry;
				}

				// Запись категорий из буфера в глобальную переменную шаблонизатора
				$this->view->categories = $category;

				// Запись товаров из буфера в глобальную переменную шаблонизатора
				$this->view->products = $product;

				// Generating filters
				$this->view->filters = [
					'brands' => product::collect(
						'd.brand.' . $this->language->name,
						errors: $this->errors['catalog']
					)
				];

				// Generating HTML and writing to the buffer of response
				$html = [
					'filters' => count($this->view->products) > 0 ? $this->view->render('catalog/elements/filters.html') : null,
					'products' => $this->view->render('catalog/elements/products/2columns.html')
				];
			}
		} else {
			// Не передана категория

			// Поиск категорий: "categories" (самый верхний уровень)
			$this->view->categories = entry::ascendants(descendant: new category, errors: $this->errors['catalog']);
		}

		if ($_SERVER['REQUEST_METHOD'] === 'GET') {
			// GET request

			// Exit (success)
			return $this->view->render('catalog/page.html');
		} else if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'title' => $title ?? '',
					'html' => $html + [
						'categories' => $this->view->render('catalog/elements/categories.html'),
					],
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Search
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function search(array $parameters = []): ?string
	{
		// Initializing of text fore search
		preg_match('/[\w\s]+/u', $parameters['text'] ?? '', $matches);
		$text = $matches[0] ?? null;

		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// Search for products
			$this->view->products = isset($text) ? product::read(
				search: $text,
				filter: 'd.deleted != true && d.hidden != true',
				sort: 'd.position ASC, d.name ASC, d.created DESC',
				amount: 30,
				language: $this->language,
				errors: $this->errors['catalog']
			) : [];

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'title' => $title ?? '',
					'html' => [
						'products' => $this->view->render('catalog/elements/products/2columns.html')
					],
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Product
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function product(array $parameters = []): ?string
	{
		// Initializing identifier of a product
		preg_match('/[\d]+/', $parameters['identifier'] ?? '', $matches);
		$identifier = $matches[0] ?? null;

		if (!empty($identifier)) {
			// Received identifier of the product

			if ($_SERVER['REQUEST_METHOD'] === 'POST') {
				// POST request

				// Search for products
				$product = product::read(
					filter: "d.identifier == @identifier && d.deleted != true && d.hidden != true",
					sort: 'd.created DESC',
					amount: 1,
					return: '{identifier: d.identifier, name: d.name.@language, description: d.description.@language, cost: d.cost, weight: d.weight, dimensions: d.dimensions, brand: d.brand.@language, compatibility: d.compatibility.@language, images: d.images[*].storage}',
					parameters: ['identifier' => (int) $identifier],
					language: $this->language,
					errors: $this->errors['catalog']
				)[0]?->getAll();

				// Initializing a response headers
				header('Content-Type: application/json');
				header('Content-Encoding: none');
				header('X-Accel-Buffering: no');

				// Initializing of the output buffer
				ob_start();

				// Generating the reponse
				echo json_encode(
					[
						'product' => $product,
						'errors' => $this->errors
					]
				);

				// Initializing a response headers
				header('Content-Length: ' . ob_get_length());

				// Sending and deinitializing of the output buffer
				ob_end_flush();
				flush();

				// Exit (success)
				return null;
			}
		}

		// Exit (fail)
		return null;
	}
}
